/*
 * This file was generated automatically by ExtUtils::ParseXS version 3.44 from the
 * contents of Splitter.xs. Do not edit this file, edit Splitter.xs instead.
 *
 *    ANY CHANGES MADE HERE WILL BE LOST!
 *
 */

#line 1 "Splitter.xs"
    /*
    ###########################################################################
    # (@)PACKAGE:Win32::GUI::Splitter
    #
    # $Id: Splitter.xs,v 1.6 2010/04/08 21:26:48 jwgui Exp $
    #
    ###########################################################################
    */

#include "GUI.h"

void 
Splitter_onPreCreate(NOTXSPROC LPPERLWIN32GUI_CREATESTRUCT perlcs) {

    perlcs->cs.lpszClass = "Win32::GUI::Splitter(vertical)";
    perlcs->cs.style = WS_VISIBLE | WS_CHILD;
    perlcs->cs.dwExStyle = WS_EX_NOPARENTNOTIFY;
}

BOOL
Splitter_onParseOption(NOTXSPROC char *option, SV* value, LPPERLWIN32GUI_CREATESTRUCT perlcs) {

    SV* storing;
    SV** stored;
    BOOL retval = TRUE;

    if(strcmp(option, "-horizontal") == 0) {
        if(SvIV(value)) {
            perlcs->cs.lpszClass = "Win32::GUI::Splitter(horizontal)";
        } else {
            perlcs->cs.lpszClass = "Win32::GUI::Splitter(vertical)";
        }
        SwitchBit(perlcs->dwPlStyle, PERLWIN32GUI_HORIZONTAL, SvIV(value));
    } else if(strcmp(option, "-min") == 0) {
        storing = newSViv((LONG) SvIV(value));
        stored = hv_store_mg(NOTXSCALL perlcs->hvSelf, "-min", 4, storing, 0);
        perlcs->iMinWidth = (int)SvIV(value);
    } else if(strcmp(option, "-max") == 0) {
        storing = newSViv((LONG) SvIV(value));
        stored = hv_store_mg(NOTXSCALL perlcs->hvSelf, "-max", 4, storing, 0);
        perlcs->iMaxWidth = (int)SvIV(value);
    } else if(strcmp(option, "-range") == 0) {
        if(SvROK(value) && SvTYPE(SvRV(value)) == SVt_PVAV) {
            SV** t;
            t = av_fetch((AV*)SvRV(value), 0, 0);
            if(t != NULL) {
                storing = newSViv((LONG) SvIV(*t));
                stored = hv_store_mg(NOTXSCALL perlcs->hvSelf, "-min", 4, storing, 0);
                perlcs->iMinWidth = (int)SvIV(*t);
            }
            t = av_fetch((AV*)SvRV(value), 1, 0);
            if(t != NULL) {
                storing = newSViv((LONG) SvIV(*t));
                stored = hv_store_mg(NOTXSCALL perlcs->hvSelf, "-max", 4, storing, 0);
                perlcs->iMaxWidth = (int)SvIV(*t);
            }
        } else {
            W32G_WARN("Win32::GUI: Argument to -range is not an array reference!");
        }
    } else retval = FALSE;

    return retval;
}

void 
Splitter_onPostCreate(NOTXSPROC HWND myhandle, LPPERLWIN32GUI_CREATESTRUCT perlcs) {
}

BOOL
Splitter_onParseEvent(NOTXSPROC char *name, int* eventID) {

    BOOL retval = TRUE;

    if Parse_Event("Release",   PERLWIN32GUI_NEM_CONTROL1)
    else retval = FALSE;

    return retval;
}

int
Splitter_onEvent (NOTXSPROC LPPERLWIN32GUI_USERDATA perlud, UINT uMsg, WPARAM wParam, LPARAM lParam) {

    int PerlResult = 0;
    POINT pt;
    HWND phwnd, hwnd;
    RECT rc;
    int w,h;

    BOOL tracking = perlud->dwPlStyle & PERLWIN32GUI_TRACKING;
    BOOL horizontal = perlud->dwPlStyle & PERLWIN32GUI_HORIZONTAL;

    switch(uMsg) {
    case WM_MOUSEMOVE:
        if(tracking) {
            hwnd  = handle_From (NOTXSCALL perlud->svSelf);
            phwnd = GetParent(hwnd);
            GetWindowRect(hwnd, &rc);
            w = rc.right - rc.left;
            h = rc.bottom - rc.top;
            ScreenToClient(phwnd, (POINT*)&rc);
            pt.x = GET_X_LPARAM(lParam);
            pt.y = GET_Y_LPARAM(lParam);
	    MapWindowPoints(hwnd, phwnd, (LPPOINT)&pt, 1);

            if(horizontal) {
                DrawSplitter(NOTXSCALL phwnd, rc.left, (int)perlud->dwData, w, h);
                pt.y = AdjustSplitterCoord(NOTXSCALL perlud, pt.y, h, phwnd);
	        perlud->dwData = (IV)(pt.y);
                DrawSplitter(NOTXSCALL phwnd, rc.left, pt.y, w, h);
            } else {
                DrawSplitter(NOTXSCALL phwnd, (int)perlud->dwData, rc.top, w, h);
                pt.x = AdjustSplitterCoord(NOTXSCALL perlud, pt.x, w, phwnd);
	        perlud->dwData = (IV)(pt.x);
                DrawSplitter(NOTXSCALL phwnd, pt.x, rc.top, w, h);
            }
        }
        break;
    case WM_LBUTTONDOWN:
        hwnd  = handle_From (NOTXSCALL perlud->svSelf);
        phwnd = GetParent(hwnd);
        GetWindowRect(hwnd, &rc);
        w = rc.right - rc.left;
        h = rc.bottom - rc.top;
        ScreenToClient(phwnd, (POINT*)&rc);
        pt.x = GET_X_LPARAM(lParam);
        pt.y = GET_Y_LPARAM(lParam);
        MapWindowPoints(hwnd, phwnd, (LPPOINT)&pt, 1);

        if(horizontal) {
            pt.y = AdjustSplitterCoord(NOTXSCALL perlud, pt.y, h, phwnd);
	    perlud->dwData = (IV)(rc.top);
            DrawSplitter(NOTXSCALL phwnd, rc.left, rc.top, w, h);
        } else {
            pt.x = AdjustSplitterCoord(NOTXSCALL perlud, pt.x, w, phwnd);
	    perlud->dwData = (IV)(rc.left);
            DrawSplitter(NOTXSCALL phwnd, rc.left, rc.top, w, h);
        }
        SwitchBit(perlud->dwPlStyle, PERLWIN32GUI_TRACKING, 1);        
        SetCapture(hwnd);
        break;
    case WM_LBUTTONUP:
        if(tracking) {
            ReleaseCapture();  // Sends us a WM_CAPTURECHANGED message
            hwnd  = handle_From (NOTXSCALL perlud->svSelf);
            phwnd = GetParent(hwnd);
            GetWindowRect(hwnd, &rc);
            w = rc.right - rc.left;
            h = rc.bottom - rc.top;
            ScreenToClient(phwnd, (POINT*)&rc);

            if(horizontal) {
                MoveWindow(hwnd, rc.left, (int)perlud->dwData, w, h, 1);
            } else {
                MoveWindow(hwnd, (int)perlud->dwData, rc.top, w, h, 1);
            }

            /*
            * (@)EVENT:Release(COORD)
            * Sent when the Splitter is released after being
            * dragged to a new location (identified by the
            * COORD parameter). COORD is the top coordinate
	    * of a horizontal splitter or the left coordinate
	    * of a vertical splitter.
            * (@)APPLIES_TO:Splitter
            */
            PerlResult = DoEvent(NOTXSCALL perlud, PERLWIN32GUI_NEM_CONTROL1, "Release",
                                 PERLWIN32GUI_ARGTYPE_LONG, perlud->dwData,
                                 -1);

        }
        break;
    case WM_CAPTURECHANGED:
        if(tracking) {
            hwnd  = handle_From (NOTXSCALL perlud->svSelf);
            phwnd = GetParent(hwnd);
            GetWindowRect(hwnd, &rc);
            w = rc.right - rc.left;
            h = rc.bottom - rc.top;
            ScreenToClient(phwnd, (POINT*)&rc);
            SwitchBit(perlud->dwPlStyle, PERLWIN32GUI_TRACKING, 0);

            if(horizontal) {
                DrawSplitter(NOTXSCALL phwnd, rc.left, (int)perlud->dwData, w, h);
            } else {
                DrawSplitter(NOTXSCALL phwnd, (int)perlud->dwData, rc.top, w, h);
            }
        }
        break;

    default :
        PerlResult = 1;
    }

    return PerlResult;
}

#line 207 "Splitter.c"
#ifndef PERL_UNUSED_VAR
#  define PERL_UNUSED_VAR(var) if (0) var = var
#endif

#ifndef dVAR
#  define dVAR		dNOOP
#endif


/* This stuff is not part of the API! You have been warned. */
#ifndef PERL_VERSION_DECIMAL
#  define PERL_VERSION_DECIMAL(r,v,s) (r*1000000 + v*1000 + s)
#endif
#ifndef PERL_DECIMAL_VERSION
#  define PERL_DECIMAL_VERSION \
	  PERL_VERSION_DECIMAL(PERL_REVISION,PERL_VERSION,PERL_SUBVERSION)
#endif
#ifndef PERL_VERSION_GE
#  define PERL_VERSION_GE(r,v,s) \
	  (PERL_DECIMAL_VERSION >= PERL_VERSION_DECIMAL(r,v,s))
#endif
#ifndef PERL_VERSION_LE
#  define PERL_VERSION_LE(r,v,s) \
	  (PERL_DECIMAL_VERSION <= PERL_VERSION_DECIMAL(r,v,s))
#endif

/* XS_INTERNAL is the explicit static-linkage variant of the default
 * XS macro.
 *
 * XS_EXTERNAL is the same as XS_INTERNAL except it does not include
 * "STATIC", ie. it exports XSUB symbols. You probably don't want that
 * for anything but the BOOT XSUB.
 *
 * See XSUB.h in core!
 */


/* TODO: This might be compatible further back than 5.10.0. */
#if PERL_VERSION_GE(5, 10, 0) && PERL_VERSION_LE(5, 15, 1)
#  undef XS_EXTERNAL
#  undef XS_INTERNAL
#  if defined(__CYGWIN__) && defined(USE_DYNAMIC_LOADING)
#    define XS_EXTERNAL(name) __declspec(dllexport) XSPROTO(name)
#    define XS_INTERNAL(name) STATIC XSPROTO(name)
#  endif
#  if defined(__SYMBIAN32__)
#    define XS_EXTERNAL(name) EXPORT_C XSPROTO(name)
#    define XS_INTERNAL(name) EXPORT_C STATIC XSPROTO(name)
#  endif
#  ifndef XS_EXTERNAL
#    if defined(HASATTRIBUTE_UNUSED) && !defined(__cplusplus)
#      define XS_EXTERNAL(name) void name(pTHX_ CV* cv __attribute__unused__)
#      define XS_INTERNAL(name) STATIC void name(pTHX_ CV* cv __attribute__unused__)
#    else
#      ifdef __cplusplus
#        define XS_EXTERNAL(name) extern "C" XSPROTO(name)
#        define XS_INTERNAL(name) static XSPROTO(name)
#      else
#        define XS_EXTERNAL(name) XSPROTO(name)
#        define XS_INTERNAL(name) STATIC XSPROTO(name)
#      endif
#    endif
#  endif
#endif

/* perl >= 5.10.0 && perl <= 5.15.1 */


/* The XS_EXTERNAL macro is used for functions that must not be static
 * like the boot XSUB of a module. If perl didn't have an XS_EXTERNAL
 * macro defined, the best we can do is assume XS is the same.
 * Dito for XS_INTERNAL.
 */
#ifndef XS_EXTERNAL
#  define XS_EXTERNAL(name) XS(name)
#endif
#ifndef XS_INTERNAL
#  define XS_INTERNAL(name) XS(name)
#endif

/* Now, finally, after all this mess, we want an ExtUtils::ParseXS
 * internal macro that we're free to redefine for varying linkage due
 * to the EXPORT_XSUB_SYMBOLS XS keyword. This is internal, use
 * XS_EXTERNAL(name) or XS_INTERNAL(name) in your code if you need to!
 */

#undef XS_EUPXS
#if defined(PERL_EUPXS_ALWAYS_EXPORT)
#  define XS_EUPXS(name) XS_EXTERNAL(name)
#else
   /* default to internal */
#  define XS_EUPXS(name) XS_INTERNAL(name)
#endif

#ifndef PERL_ARGS_ASSERT_CROAK_XS_USAGE
#define PERL_ARGS_ASSERT_CROAK_XS_USAGE assert(cv); assert(params)

/* prototype to pass -Wmissing-prototypes */
STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params);

STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params)
{
    const GV *const gv = CvGV(cv);

    PERL_ARGS_ASSERT_CROAK_XS_USAGE;

    if (gv) {
        const char *const gvname = GvNAME(gv);
        const HV *const stash = GvSTASH(gv);
        const char *const hvname = stash ? HvNAME(stash) : NULL;

        if (hvname)
	    Perl_croak_nocontext("Usage: %s::%s(%s)", hvname, gvname, params);
        else
	    Perl_croak_nocontext("Usage: %s(%s)", gvname, params);
    } else {
        /* Pants. I don't think that it should be possible to get here. */
	Perl_croak_nocontext("Usage: CODE(0x%" UVxf ")(%s)", PTR2UV(cv), params);
    }
}
#undef  PERL_ARGS_ASSERT_CROAK_XS_USAGE

#define croak_xs_usage        S_croak_xs_usage

#endif

/* NOTE: the prototype of newXSproto() is different in versions of perls,
 * so we define a portable version of newXSproto()
 */
#ifdef newXS_flags
#define newXSproto_portable(name, c_impl, file, proto) newXS_flags(name, c_impl, file, proto, 0)
#else
#define newXSproto_portable(name, c_impl, file, proto) (PL_Sv=(SV*)newXS(name, c_impl, file), sv_setpv(PL_Sv, proto), (CV*)PL_Sv)
#endif /* !defined(newXS_flags) */

#if PERL_VERSION_LE(5, 21, 5)
#  define newXS_deffile(a,b) Perl_newXS(aTHX_ a,b,file)
#else
#  define newXS_deffile(a,b) Perl_newXS_deffile(aTHX_ a,b)
#endif

#line 351 "Splitter.c"
#pragma message( "*** PACKAGE Win32::GUI::Splitter..." )

XS_EUPXS(XS_Win32__GUI__Splitter_Min); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Win32__GUI__Splitter_Min)
{
    dVAR; dXSARGS;
    if (items < 1)
       croak_xs_usage(cv,  "handle, ...");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
	HWND	handle;
#line 213 "Splitter.xs"
    LPPERLWIN32GUI_USERDATA perlud;
#line 366 "Splitter.c"

    if(SvROK(ST(0))) {
        SV** out=hv_fetch((HV*)SvRV(ST(0)), "-handle", 7, 0);
        if(out != NULL)
            handle = INT2PTR(HWND,SvIV(*out));
        else
            handle = NULL;
    } else
       handle = INT2PTR(HWND,SvIV(ST(0)))
;
#line 215 "Splitter.xs"
    if(items > 2) {
        CROAK("Usage: Min(handle, [value]);\n");
    }
    perlud = (LPPERLWIN32GUI_USERDATA) GetWindowLongPtr(handle, GWLP_USERDATA);
    if( ValidUserData(perlud) ) {
        if(items == 1) {
            XSRETURN_IV(perlud->iMinWidth);
        } else {
            perlud->iMinWidth = (int)SvIV(ST(1));
            XSRETURN_YES;
        }
    } else {
        XSRETURN_UNDEF;
    }
#line 392 "Splitter.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Win32__GUI__Splitter_Max); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Win32__GUI__Splitter_Max)
{
    dVAR; dXSARGS;
    if (items < 1)
       croak_xs_usage(cv,  "handle, ...");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
	HWND	handle;
#line 240 "Splitter.xs"
    LPPERLWIN32GUI_USERDATA perlud;
#line 411 "Splitter.c"

    if(SvROK(ST(0))) {
        SV** out=hv_fetch((HV*)SvRV(ST(0)), "-handle", 7, 0);
        if(out != NULL)
            handle = INT2PTR(HWND,SvIV(*out));
        else
            handle = NULL;
    } else
       handle = INT2PTR(HWND,SvIV(ST(0)))
;
#line 242 "Splitter.xs"
    if(items > 2) {
        CROAK("Usage: Max(handle, [value]);\n");
    }
    perlud = (LPPERLWIN32GUI_USERDATA) GetWindowLongPtr(handle, GWLP_USERDATA);
    if( ValidUserData(perlud) ) {
        if(items == 1) {
            XSRETURN_IV(perlud->iMaxWidth);
        } else {
            perlud->iMaxWidth = (int)SvIV(ST(1));
            XSRETURN_YES;
        }
    } else {
        XSRETURN_UNDEF;
    }
#line 437 "Splitter.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Win32__GUI__Splitter_Horizontal); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Win32__GUI__Splitter_Horizontal)
{
    dVAR; dXSARGS;
    if (items < 1)
       croak_xs_usage(cv,  "handle, ...");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
	HWND	handle;
#line 267 "Splitter.xs"
    LPPERLWIN32GUI_USERDATA perlud;
#line 456 "Splitter.c"

    if(SvROK(ST(0))) {
        SV** out=hv_fetch((HV*)SvRV(ST(0)), "-handle", 7, 0);
        if(out != NULL)
            handle = INT2PTR(HWND,SvIV(*out));
        else
            handle = NULL;
    } else
       handle = INT2PTR(HWND,SvIV(ST(0)))
;
#line 269 "Splitter.xs"
    if(items > 2) {
        CROAK("Usage: Horizontal(handle, [value]);\n");
    }
    perlud = (LPPERLWIN32GUI_USERDATA) GetWindowLongPtr(handle, GWLP_USERDATA);
    if( ValidUserData(perlud) ) {
        if(items == 1) {
            XSRETURN_IV(perlud->dwPlStyle & PERLWIN32GUI_HORIZONTAL);
        } else {
            SwitchBit(perlud->dwPlStyle, PERLWIN32GUI_HORIZONTAL, SvIV(ST(1)));
            SetWindowLongPtr(handle, GWLP_USERDATA, (LONG_PTR) perlud);
            XSRETURN_YES;
        }
    } else {
        XSRETURN_UNDEF;
    }
#line 483 "Splitter.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Win32__GUI__Splitter_Vertical); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Win32__GUI__Splitter_Vertical)
{
    dVAR; dXSARGS;
    if (items < 1)
       croak_xs_usage(cv,  "handle, ...");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
	HWND	handle;
#line 294 "Splitter.xs"
    LPPERLWIN32GUI_USERDATA perlud;
#line 502 "Splitter.c"

    if(SvROK(ST(0))) {
        SV** out=hv_fetch((HV*)SvRV(ST(0)), "-handle", 7, 0);
        if(out != NULL)
            handle = INT2PTR(HWND,SvIV(*out));
        else
            handle = NULL;
    } else
       handle = INT2PTR(HWND,SvIV(ST(0)))
;
#line 296 "Splitter.xs"
    if(items > 2) {
        CROAK("Usage: Vertical(handle, [value]);\n");
    }
    perlud = (LPPERLWIN32GUI_USERDATA) GetWindowLongPtr(handle, GWLP_USERDATA);
    if( ValidUserData(perlud) ) {
        if(items == 1) {
            XSRETURN_IV(!(perlud->dwPlStyle & PERLWIN32GUI_HORIZONTAL));
        } else {
            SwitchBit(perlud->dwPlStyle, PERLWIN32GUI_HORIZONTAL, !SvIV(ST(1)));
            SetWindowLongPtr(handle, GWLP_USERDATA, (LONG_PTR) perlud);
            XSRETURN_YES;
        }
    } else {
        XSRETURN_UNDEF;
    }
#line 529 "Splitter.c"
	PUTBACK;
	return;
    }
}

#ifdef __cplusplus
extern "C"
#endif
XS_EXTERNAL(boot_Win32__GUI__Splitter); /* prototype to pass -Wmissing-prototypes */
XS_EXTERNAL(boot_Win32__GUI__Splitter)
{
#if PERL_VERSION_LE(5, 21, 5)
    dVAR; dXSARGS;
#else
    dVAR; dXSBOOTARGSXSAPIVERCHK;
#endif
#if PERL_VERSION_LE(5, 8, 999) /* PERL_VERSION_LT is 5.33+ */
    char* file = __FILE__;
#else
    const char* file = __FILE__;
#endif

    PERL_UNUSED_VAR(file);

    PERL_UNUSED_VAR(cv); /* -W */
    PERL_UNUSED_VAR(items); /* -W */
#if PERL_VERSION_LE(5, 21, 5)
    XS_VERSION_BOOTCHECK;
#  ifdef XS_APIVERSION_BOOTCHECK
    XS_APIVERSION_BOOTCHECK;
#  endif
#endif

        newXS_deffile("Win32::GUI::Splitter::Min", XS_Win32__GUI__Splitter_Min);
        newXS_deffile("Win32::GUI::Splitter::Max", XS_Win32__GUI__Splitter_Max);
        newXS_deffile("Win32::GUI::Splitter::Horizontal", XS_Win32__GUI__Splitter_Horizontal);
        newXS_deffile("Win32::GUI::Splitter::Vertical", XS_Win32__GUI__Splitter_Vertical);
#if PERL_VERSION_LE(5, 21, 5)
#  if PERL_VERSION_GE(5, 9, 0)
    if (PL_unitcheckav)
        call_list(PL_scopestack_ix, PL_unitcheckav);
#  endif
    XSRETURN_YES;
#else
    Perl_xs_boot_epilog(aTHX_ ax);
#endif
}

